@@ -11,6 +11,7 @@ import gobject
 import math
 import gobject
 import pygst
+import urllib2
 import email
 from subprocess import PIPE, Popen
 from threading import Thread
@@ -39,9 +40,9 @@ except ImportError:
 ON_POSIX = 'posix' in sys.builtin_module_names
 
 def enqueue_output(out, queue):
-    for line in iter(out.readline, ''):
-        queue.put(line)
-    out.close()
+	for line in iter(out.readline, ''):
+		queue.put(line)
+	out.close()
 
 pulsing = False
 rot_amount = 0
@@ -59,7 +60,14 @@ settings['use_google_voice'] = "unrecognized"
 settings['language'] = 'english'
 lang = "en"
 lang_flag = "wiki"
+# Levels:
+#	0: No accuracy. User is probably somewhere on Earth.
+#	1: Very low accuracy, using geoip on cellular connection - accurate to within a thousand miles or so
+#	2: Medium accuracy, using geoip with wifi connection - accurate to within 10 miles
+#	3: High accuracy, using GPS - accurate to within 20 feet
+POSITION_ACCURACY = 0
 gv_api_url='http://www.google.com/speech-api/v1/recognize?lang='+lang+'-'+lang+'&client=chromium'
+ipinfodb_api_key = '799619bbf0ddff5c7b3897e12a0289849103af68e7bf6e348db653dba1ced573'
 SAMPLE_RATE = 16000
 try:
 	parsed_settings = json.loads(open(os.getenv('HOME')+'/.saera').read())
@@ -76,6 +84,45 @@ try:
 except OSError:
 	pass
 
+def get_ip_location():
+	global POSITION_ACCURACY
+	if POSITION_ACCURACY<3:
+		try:
+			response = urllib2.urlopen("http://api.ipinfodb.com/v3/ip-city/?key="+ipinfodb_api_key).read()
+			status, huh, ip, cc, country, state, city, zip, lat, lon, tz = response.split(';')
+			settings['state'] = state
+			settings['city'] = city
+			settings['zip'] = zip
+			settings['lat'] = lat
+			settings['lon'] = lon
+			saera_processing.location = (lat, lon)
+			POSITION_ACCURACY = 2
+		except urllib2.URLError:
+			# Probably not connected to internet
+			pass
+
+def get_geo_location():
+	global POSITION_ACCURACY
+	if POSITION_ACCURACY<3:
+		try:
+			import location
+			control = location.GPSDControl.get_default()
+			device = location.GPSDevice()
+			control.set_properties(preferred_method=location.METHOD_ACWP)
+			def on_changed(device, user_data):
+				if not device:
+					return
+				if device.fix:
+					if device.fix[1] & location.GPS_DEVICE_LATLONG_SET:
+						settings['lat'], settings['lon'] = device.fix[4:6]
+						saera_processing.location = (lat, lon)
+					print "horizontal accuracy: %f meters" % (device.fix[6] / 100)
+					POSITION_ACCURACY = 2
+			device.connect("changed", on_changed)
+			control.start()
+		except ImportError:
+			get_ip_location()
+
 class MailFolder:
 	def __init__(self, foldername, foldertype):
 		self.foldername = foldername
@@ -151,6 +198,9 @@ class Saera:
 
 		self.init_gst()
 		self.init_mail()
+
+		get_ip_location()
+		get_geo_location()
 		
 		# Commented out because there is a bug in the sqlite line that I can't find.
 		gobject.timeout_add(10000, self.check_texts)
@@ -264,7 +314,10 @@ class Saera:
 				try:
 					self.run_saera(None, 'speech-event', ' ')
 				finally:
-					os.remove('/tmp/saera/output.flac')
+					fsink = self.pipeline.get_by_name('fsink')
+					fsink.set_state(gst.STATE_NULL)
+					# os.remove('/tmp/saera/output.flac')
+					fsink.set_state(gst.STATE_READY)
 					able_to_listen = True
 
 	def check_proximity_sensor(self):
@@ -451,7 +504,7 @@ class Saera:
 				cbox.append_text(i)
 			vbox.pack_start(cbox, True, False, 0)
 		win.show_all()
-    
+	
 
 	def run_saera(self, widget=None, event=None, data=None):
 		try:
@@ -573,7 +626,7 @@ class Saera:
 										 # + '! vader name=vad auto-threshold=true '
 										 # # + '! pocketsphinx name=asr ! fakesink')
 										 # + '! pocketsphinx name=asr t. ! queue ! audiorate ! audio/x-raw-int, rate=48000 ! flacenc'
-										 + ' ! filesink location=/tmp/saera/output.flac')
+										 + ' ! filesink name=fsink location=/tmp/saera/output.flac')
 		asr = self.pipeline.get_by_name('asr')
 		asr.connect('partial_result', self.asr_partial_result)
 		asr.connect('result', self.asr_result)
@@ -625,7 +678,13 @@ class Saera:
 		"""Insert the final result."""
 		# All this stuff appears as one single action
 		print "Final Result:", hyp
-		self.run_saera(None, "speech-event", hyp)
+		try:
+			self.run_saera(None, "speech-event", hyp)
+		finally:
+			fsink = self.pipeline.get_by_name('fsink')
+			fsink.set_state(gst.STATE_NULL)
+			os.remove('/tmp/saera/output.flac')
+			fsink.set_state(gst.STATE_READY)
 
 if __name__=='__main__':
 
