@@ -299,16 +299,19 @@ def take_picture(input):
 			return sent.took_picture, 'self.display_picture("/home/user/MyDocs/DCIM/'+photoname+num+'.jpg")'
 
 def where_am_i(input):
-	return sent.heres_where_you_are, "gobject.timeout_add(500, os.system,'python /opt/modrana/modrana.py --center-on-position')"
+	return sent.heres_where_you_are, "gobject.timeout_add(500, os.system,'python /opt/modrana/modrana.py --focus-on-coordinates geo:"+ \
+		str(location[0])+","+str(location[1])+"')"
 
 def food(input):
 	food = list(set(sent.food).intersection(set(input.split())))
 	food = food[0] if food else 'food'
 	#TODO - cmdline
-	return 'Let me search for '+food, "gobject.timeout_add(500, os.system, 'python /opt/modrana/modrana.py --local-search "+food+"')"
+	return 'Let me search for '+food, "gobject.timeout_add(500, os.system, 'python /opt/modrana/modrana.py --local-search-location geo:" + \
+		str(location[0])+","+str(location[1])+" --local-search "+food+"')"
 
 def gas(input):
-	return 'Let me search for gas stations.', "gobject.timeout_add(500, os.system, 'python /opt/modrana/modrana.py --local-search gas')"
+	return 'Let me search for gas stations.', "gobject.timeout_add(500, os.system, 'python /opt/modrana/modrana.py --local-search-location geo:" + \
+		str(location[0])+","+str(location[1])+" --local-search gas')"
 
 def get_weather(input):
 	try:
